@@ -147,23 +147,36 @@ class PortfolioDataApi(metaclass=Singleton):
             print(f'Failed to get data <err_msg> {e} from portfolio.get_part_fund_info')
 
 
-    def get_all_fund_nav(self, start_date='', end_date='', fund_list: Tuple[str] = ()):
+    def get_all_fund_nav(self, start_date=None, end_date=None, fund_list: Tuple[str] = ()):
         try:
-            if fund_list is not None:
+            if start_date is None:
+                start_date = ''
+            else:
+                start_date = start_date-datetime.timedelta(days=20)
+            end_date = end_date if end_date is not None else ''
+            mutual_fund_nav = pd.DataFrame()
+            hedge_fund_nav = pd.DataFrame
+            #print(f'start_date {start_date} end_date {end_date} fund_list {fund_list}')
+            if len(fund_list) > 0:
                 mutual_fund_list = [_ for _ in fund_list if '!' in _]
                 hedge_fund_list = [_ for _ in fund_list if '!' not in _]
-                mutual_fund_nav = BasicDataApi().get_fund_nav_with_date(start_date=start_date-datetime.timedelta(days=20),end_date=end_date,fund_list=fund_list)
-                hedge_fund_nav = RawDataApi().get_hf_fund_nav(fund_ids=hedge_fund_list, start_date=start_date-datetime.timedelta(days=20), end_date=end_date)
+                if len(mutual_fund_list) > 0:
+                    mutual_fund_nav = BasicDataApi().get_fund_nav_with_date(start_date=start_date,end_date=end_date,fund_list=mutual_fund_list)
+                if len(hedge_fund_list) > 0:
+                    hedge_fund_nav = RawDataApi().get_hf_fund_nav(fund_ids=hedge_fund_list, start_date=start_date, end_date=end_date)
             else:
-                mutual_fund_nav = BasicDataApi().get_fund_nav_with_date(start_date=start_date-datetime.timedelta(days=20),end_date=end_date)
-                hedge_fund_nav = RawDataApi().get_hf_fund_nav_dt(start_date=start_date-datetime.timedelta(days=20))
+                mutual_fund_nav = BasicDataApi().get_fund_nav_with_date(start_date=start_date,end_date=end_date)
+                hedge_fund_nav = RawDataApi().get_hf_fund_nav_dt(start_date=start_date, end_date=end_date)
             
             if not mutual_fund_nav.empty:
                 mutual_fund_nav = mutual_fund_nav.pivot_table(index='datetime',columns='fund_id',values='adjusted_net_value')
             if not hedge_fund_nav.empty:
                 hedge_fund_nav = hedge_fund_nav.pivot_table(index='datetime',columns='fund_id',values='nav')
             df_list = [i for i in [mutual_fund_nav, hedge_fund_nav] if not i.empty ]
-            fund_nav = pd.concat(df_list, axis=1).sort_index().ffill().loc[start_date:]
+            if isinstance(start_date, str):
+                fund_nav = pd.concat(df_list, axis=1).sort_index().ffill()
+            else:
+                fund_nav = pd.concat(df_list, axis=1).sort_index().ffill().loc[start_date:]
             fund_nav.index.name = 'datetime'
             return fund_nav
 
@@ -202,13 +215,18 @@ class PortfolioDataApi(metaclass=Singleton):
             fund_nav = self.get_all_fund_nav(start_date=_begin_date,fund_list=fund_list)
             if fund_nav.empty:
                 print('not fund nav existed')
-                return {
+                data = {
                     '净值':pd.DataFrame(),
                     '各基净值':pd.DataFrame(),
                     '各基权重':pd.DataFrame(),
                     '大类净值':pd.DataFrame(),
                     '持有信息':{},
                 }
+                data['净值'].index.name = 'datetime'
+                data['各基净值'].index.name = 'datetime'
+                data['各基权重'].index.name = 'datetime'
+                data['大类净值'].index.name = 'datetime'
+                return data
             fund_nav = fund_nav.ffill()
             order_dic = { dt: i[dt] for i in order_list for dt in list(i.keys())}
             date_list = [j for i in order_list for j in i.keys()] + [fund_nav.index[-1]]
@@ -221,7 +239,9 @@ class PortfolioDataApi(metaclass=Singleton):
             for dts in date_pairs:
                 b_d = dts[0]
                 e_d = dts[1]
+                
                 fund_wgt_dt = pd.Series({k : v for k, v in order_dic[b_d].items() if v > 0})
+                fund_wgt_dt = fund_wgt_dt / fund_wgt_dt.sum()
                 fund_amt_dt = port_amount * fund_wgt_dt
                 fund_nav_dt = fund_nav.loc[b_d:e_d,fund_wgt_dt.keys()]
                 fund_vol_dt = port_amount * fund_wgt_dt / fund_nav_dt.iloc[0]
@@ -236,17 +256,23 @@ class PortfolioDataApi(metaclass=Singleton):
                 port_mv_result.append(port_mv)
                 fund_type_result = []
                 for fund_type_i, fund_list_i in fund_type.items():
-                    fund_mv_type_i = fund_amount_dt[fund_amount_dt.columns.intersection(fund_list_i)].sum(axis=1).rename(fund_type_i).to_frame()
+                    _fund_list = list(set(fund_list_i).intersection(fund_wgt_dt.index.tolist()))
+                    fund_mv_type_i = fund_amount_dt[_fund_list].sum(axis=1).rename(fund_type_i).to_frame()
+                    fund_mv_type_i = fund_mv_type_i.pct_change(1).iloc[1:]
                     fund_type_result.append(fund_mv_type_i)
                 fund_type_amt_dt = pd.concat(fund_type_result, axis=1)
                 port_type_result.append(fund_type_amt_dt)
             port_mv_result = [i.iloc[:-1] if i.index[-1] != port_mv_result[-1].index[-1] else i for i in port_mv_result]
             port_mv_result = pd.concat(port_mv_result)
+            port_mv_result = port_mv_result.reindex(pd.date_range(start=port_mv_result.index[0], end=port_mv_result.index[-1]))
+            port_mv_result.index = [i.date() for i in port_mv_result.index]
             port_mv_result.index.name = 'datetime'
             port_weight_result = [i.iloc[:-1] if i.index[-1] != port_weight_result[-1].index[-1] else i for i in port_weight_result]
             port_weight_result = pd.concat(port_weight_result)
-            port_type_result = [i.iloc[:-1] if i.index[-1] != port_type_result[-1].index[-1] else i for i in port_type_result]
             port_type_result = pd.concat(port_type_result)
+            port_type_result = (port_type_result.fillna(0) + 1).cumprod()
+            port_type_result.loc[_begin_date,:] = 1
+            port_type_result = port_type_result.sort_index()
             data = {
                 '净值':port_mv_result,
                 '各基净值':fund_nav,
@@ -257,6 +283,18 @@ class PortfolioDataApi(metaclass=Singleton):
             return data
         except Exception as e:
             print(f'Failed to get data <err_msg> {e} from portfolio.get_portfolio_nav')
+            data = {
+                '净值':pd.DataFrame(),
+                '各基净值':pd.DataFrame(),
+                '各基权重':pd.DataFrame(),
+                '大类净值':pd.DataFrame(),
+                '持有信息':{},
+            }
+            data['净值'].index.name = 'datetime'
+            data['各基净值'].index.name = 'datetime'
+            data['各基权重'].index.name = 'datetime'
+            data['大类净值'].index.name = 'datetime'
+            return data
    
     def get_portfolio_benchmark_info(self):
         try:
@@ -268,6 +306,7 @@ class PortfolioDataApi(metaclass=Singleton):
 
         except Exception as e:
             print(f'Failed to get data <err_msg> {e} from portfolio.get_portfolio_benchmark_info')
+            return []
 
     def get_portfolio_mv(self, fund_nav, index_id, begin_date, end_date, time_para):
         try:
@@ -276,12 +315,12 @@ class PortfolioDataApi(metaclass=Singleton):
                 df.index.name = 'datetime'
                 data = {
                     'data':df,
-                    'mdd_1':{},
                     'stats':[],
                 }
                 return data
             def print_st(mdd):
                 return str(round(mdd * 100,2)) + '%' 
+            fund_nav = fund_nav.dropna()
             fund_nav.columns=['组合']
             basic = BasicDataApi()
             index_price = basic.get_index_price_dt(index_list=[index_id],start_date=fund_nav.index[0] - datetime.timedelta(days=10),end_date=fund_nav.index[-1]).pivot_table(index='datetime',columns='index_id',values='close')
@@ -334,6 +373,13 @@ class PortfolioDataApi(metaclass=Singleton):
             return data
         except Exception as e:
             print(f'Failed to get data <err_msg> {e} from portfolio.get_portfolio_mv')
+            df = pd.DataFrame()
+            df.index.name = 'datetime'
+            data = {
+                'data':df,
+                'stats':[],
+            }
+            return data
 
     def get_portfolio_cur_mdd(self, fund_nav, begin_date, end_date, time_para):
         try:        
@@ -341,7 +387,8 @@ class PortfolioDataApi(metaclass=Singleton):
                 data = {
                     '历史回撤':pd.DataFrame(),
                     '最低值':None,
-                }    
+                }
+                data['历史回撤'].index.name = 'datetime'
                 return data
             begin_date, end_date = RawDataApi().get_date_range(time_para, begin_date, end_date)
             fund_nav = fund_nav.loc[begin_date:end_date]
@@ -356,6 +403,12 @@ class PortfolioDataApi(metaclass=Singleton):
             return data
         except Exception as e:
             print(f'Failed to get data <err_msg> {e} from portfolio.get_portfolio_cur_mdd')
+            data = {
+                '历史回撤':pd.DataFrame(),
+                '最低值':None,
+            }
+            data['历史回撤'].index.name = 'datetime'
+            return data
 
     def get_portfolio_ret_distribution(self, fund_nav, period, begin_date, end_date, time_para):
         try:
@@ -365,6 +418,7 @@ class PortfolioDataApi(metaclass=Singleton):
                     '分布_收益':[],
                     '收益':pd.DataFrame(),
                 }
+                data['收益'].index.name = 'datetime'
                 return data
             begin_date, end_date = RawDataApi().get_date_range(time_para, begin_date, end_date)
             fund_nav = fund_nav.loc[begin_date:end_date]    
@@ -380,14 +434,22 @@ class PortfolioDataApi(metaclass=Singleton):
                 '收益':ret
             }
             return data
+
         except Exception as e:
-            print(f'Failed to get data <err_msg> {e} from portfolio.get_portfolio_cur_mdd')
+            print(f'Failed to get data <err_msg> {e} from portfolio.get_portfolio_ret_distribution')
+            data = {
+                '分布_频度':[],
+                '分布_收益':[],
+                '收益':pd.DataFrame(),
+            }
+            data['收益'].index.name = 'datetime'
+            return data
 
     def get_portfolio_stats(self, fund_nav, index_id):
         try:
             if fund_nav.empty:
-                return pd.DataFrame()
-
+                return None
+            fund_nav = fund_nav.dropna()
             fund_nav.columns=['组合']
             basic = BasicDataApi()
             index_price = basic.get_index_price_dt(index_list=[index_id],start_date=fund_nav.index[0] - datetime.timedelta(days=10),end_date=fund_nav.index[-1]).pivot_table(index='datetime',columns='index_id',values='close')
@@ -507,6 +569,7 @@ class PortfolioDataApi(metaclass=Singleton):
         
         except Exception as e:
             print(f'Failed to get data <err_msg> {e} from portfolio.get_rolling_corr')
+            return pd.DataFrame()
     
     def get_port_fund_list(self, trade_history):
         try:
@@ -521,6 +584,7 @@ class PortfolioDataApi(metaclass=Singleton):
             return result
         except Exception as e:
             print(f'Failed to get data <err_msg> {e} from portfolio.get_port_fund_list')
+            return []
 
     def get_port_fund_pos_date(self,fund_id, trade_history, index_id): 
         try:
@@ -528,13 +592,17 @@ class PortfolioDataApi(metaclass=Singleton):
             basic = BasicDataApi()
             fund_list = [fund_id]
             fund_info = self.get_part_fund_info(fund_list=fund_list)
-            desc_name_dic = fund_info.set_index('fund_id').to_dict()['desc_name']
-
+            desc_name_dic = fund_info.set_index('fund_id')['desc_name'].to_dict()
+            index_info = BasicDataApi().get_index_info(index_list=[index_id])
+            index_desc_name_dic = index_info.set_index('index_id')['desc_name'].to_dict()
+            desc_name_dic.update(index_desc_name_dic)
             begin_date, end_date = RawDataApi().get_date_range(time_para='ALL', begin_date=None, end_date=None)
             fund_nav = self.get_all_fund_nav(fund_list=[fund_id])
             if fund_nav.empty:
+                df = pd.DataFrame()
+                df.index.name = 'datetime'
                 data = {
-                '净值':pd.DataFrame(),
+                '净值':df,
                 '持有期':[],
                 }
                 return data
@@ -578,6 +646,13 @@ class PortfolioDataApi(metaclass=Singleton):
             return data
         except Exception as e:
             print(f'Failed to get data <err_msg> {e} from portfolio.get_port_fund_pos_date')
+            df = pd.DataFrame()
+            df.index.name = 'datetime'
+            data = {
+                '净值':df,
+                '持有期':[],
+                }
+            return data
 
     def get_port_ret_resolve_date_range(self, trade_history):
         try:
@@ -588,11 +663,12 @@ class PortfolioDataApi(metaclass=Singleton):
             return data
         except Exception as e:
             print(f'Failed to get data <err_msg> {e} from portfolio.get_port_ret_resolve_date_range')
+            return {}
     
     def get_port_ret_resolve(self, share_list , begin_date , end_date):
         try:
             if len(share_list) < 1:
-                return []
+                return {}
             basic = BasicDataApi()
             fund_list = list(set([i for dt, share_list in share_list.items() for i in list(share_list.keys())]))
             fund_info = self.get_all_fund_info(fund_list=fund_list)
@@ -626,9 +702,13 @@ class PortfolioDataApi(metaclass=Singleton):
                     fund_ret = round((fund_nav.iloc[-1].values[0] * fund_share - fund_nav.iloc[0].values[0] * fund_share)*100,2)
                     ret_dict[desc_name_dic[fund_id]] = fund_ret
                 all_ret_dict.append(ret_dict)
-            return all_ret_dict
+                dic = ((pd.DataFrame(all_ret_dict).T.fillna(0)+1).prod(axis=1)-1).to_dict()
+                dic = {k:round(v,2) for k,v in dic.items()}
+
+            return dic
         except Exception as e:
             print(f'Failed to get data <err_msg> {e} from portfolio.get_port_ret_resolve')
+            return {}
 
     def get_port_fund_weight(self, fund_wgt, trade_history):
         try:
@@ -661,6 +741,8 @@ class PortfolioDataApi(metaclass=Singleton):
             index_list = trade_list + [fund_wgt.index[0]] + [fund_wgt.index[-1]]
             fund_wgt_part_2 = fund_wgt.loc[fund_wgt.index.intersection(index_list)]
             fund_wgt = pd.concat([fund_wgt_part_1,fund_wgt_part_2]).sort_index()
+            fund_wgt = fund_wgt.drop_duplicates()
+            fund_type_wgt = fund_type_wgt.reindex(fund_wgt.index)
             data = {
                 '基金':fund_wgt,
                 '类型':fund_type_wgt,
@@ -669,16 +751,25 @@ class PortfolioDataApi(metaclass=Singleton):
             return data
         except Exception as e:
             print(f'Failed to get data <err_msg> {e} from portfolio.get_port_fund_weight')
+            data = {
+                '基金':pd.DataFrame(),
+                '类型':pd.DataFrame(),
+                '类型对照':{},
+            }
+            return data
+
 
     def get_port_fund_mdd_details(self, trade_history):
         try:
             order_list, fund_type = self.trade_history_parse(trade_history)
             result = self.get_portfolio_nav(trade_history=trade_history)
             df = result['净值']
-            if data.empty:
+            if df.empty:
+                df = pd.DataFrame()
+                df.index.name ='datetime'
                 data = {
-                    'mdd': pd.DataFrame(),
-                    'fund_nav': pd.DataFrame(),
+                    'mdd': df,
+                    'fund_nav': df,
                 }
                 return data
 
@@ -695,7 +786,7 @@ class PortfolioDataApi(metaclass=Singleton):
             mdd_date2 = mdd_part1.idxmin()
             exchange_list = [j for i in order_list for j in list(i.keys()) ]
             exchange_list = np.array(exchange_list)
-            exchange_dts = exchange_list[(exchange_list> mdd_date1) & (exchange_list < mdd_date2)]
+            exchange_dts = exchange_list[(exchange_list> mdd_date1) & (exchange_list < mdd_date2)].tolist()
             fund_list = list(set([z for i in order_list for j in i.values() for z in j.keys() ]))
             if len(exchange_dts) == 0:
                 date_list = [[mdd_date1,mdd_date2]]
@@ -710,7 +801,7 @@ class PortfolioDataApi(metaclass=Singleton):
                 e_d = date_group[1]
                 fund_wgt_dt = fund_wgt.loc[b_d].dropna()
                 mdd_i = ((fund_nav.loc[e_d] / fund_nav.loc[b_d] - 1) * fund_wgt_dt)
-                _mdd_list.append(mdd_i)
+                _mdd_list.append(mdd_i.dropna())
             mdd_dict = ((pd.concat(_mdd_list,axis=1).fillna(0) + 1).prod(axis=1) - 1).to_dict()
             df_result = pd.DataFrame([desc_name_dic]).T
             df_result.columns = ['基金名']
@@ -720,7 +811,7 @@ class PortfolioDataApi(metaclass=Singleton):
             df_result.index.name = '基金代码'
             df_result = df_result.reset_index()
 
-            fund_nav = fund_nav.loc[mdd_date1:mdd_date2].rename(columns=desc_name_dic)
+            fund_nav = fund_nav.loc[mdd_date1:mdd_date2][df_result['基金代码']].rename(columns=desc_name_dic)
             fund_nav = fund_nav.reindex(pd.date_range(start=fund_nav.index[0], end=fund_nav.index[-1]))
             fund_nav.index.name = 'datetime'
             data = {
@@ -731,6 +822,13 @@ class PortfolioDataApi(metaclass=Singleton):
 
         except Exception as e:
             print(f'Failed to get data <err_msg> {e} from portfolio.get_port_fund_mdd_details')
+            df = pd.DataFrame()
+            df.index.name ='datetime'
+            data = {
+                'mdd': df,
+                'fund_nav': df,
+            }
+            return data
 
     def get_port_rolling_beta_index_info(self):
         return [['hs300','沪深300'],['csi500','中证500']] 
@@ -741,8 +839,8 @@ class PortfolioDataApi(metaclass=Singleton):
             result = PortfolioDataApi().get_portfolio_nav(trade_history=trade_history)
             fund_nav = result['各基净值']
             fund_wgt = result['各基权重']
-            if fund_nav.empty():
-                return []
+            if fund_nav.empty:
+                return {}
             
             dts_list = [j for i in order_list for j in i.keys()] + [fund_nav.index[-1]]
             dts_list = [[i,dts_list[dts_list.index(i)+1]] for i in dts_list if i != dts_list[-1]]
@@ -790,6 +888,7 @@ class PortfolioDataApi(metaclass=Singleton):
 
         except Exception as e:
             print(f'Failed to get data <err_msg> {e} from portfolio.get_port_fund_alpha_detail')
+            return {}
 
     def get_port_recent_ret_details(self, trade_history, index_id, time_para, begin_date, end_date, benchmark_id_info):
         try:
@@ -798,13 +897,26 @@ class PortfolioDataApi(metaclass=Singleton):
             result = PortfolioDataApi().get_portfolio_nav(trade_history=trade_history)
             fund_nav=result['净值']
             if fund_nav.empty:
+                df = pd.DataFrame()
+                df.index.name = 'datetime'
                 data = {
-                '组合收益':pd.DataFrame(),
+                '组合收益':df,
                 '组合收益描述':[],
-                '资产价格比收益':pd.DataFrame(),
+                '资产价格比收益':df,
                 '资产价格比收益描述':[],
-                '个基近期收益':pd.DataFrame(),
+                '个基近期收益':df,
             }
+            # 取非0权重大类
+            fund_wgt = result['各基权重'].tail()
+            _result = []
+            for type_i, fund_list in fund_type.items():
+                _df = fund_wgt[fund_list].sum(axis=1)
+                _df.name = type_i
+                _result.append(_df)
+            fund_type_wgt = pd.concat(_result,axis=1)
+            fund_type_wgt = fund_type_wgt.tail(1).reset_index(drop=True).T
+            exsited_index_list = fund_type_wgt[fund_type_wgt>0].dropna().index.tolist()
+
             port_nav = PortfolioDataApi().get_portfolio_mv(fund_nav=fund_nav, index_id=index_id,begin_date=begin_date,end_date=end_date,time_para=time_para)
             fund_info = self.get_all_fund_info(fund_list = result['各基净值'].columns.tolist())
             fund_desc_dic = fund_info.set_index('fund_id')['desc_name'].to_dict()
@@ -823,6 +935,8 @@ class PortfolioDataApi(metaclass=Singleton):
             index_price = index_price.rename(columns=benchmark_id_info_r)
             index_price = index_price.reindex(_index_price.index)
             result_2 = _index_price / _index_price.iloc[0] / (index_price / index_price.iloc[0])
+            # if remove not existed asset
+            #result_2 = result_2[exsited_index_list]
             price_ratio = result_2.copy()
             res_2_dic = (_index_price.iloc[-1] / _index_price.iloc[0] - 1) * 10000
             result_2_str = ' '.join([f'{k} {int(v)}BP' for k, v in res_2_dic.items()])
@@ -857,3 +971,13 @@ class PortfolioDataApi(metaclass=Singleton):
             return data
         except Exception as e:
             print(f'Failed to get data <err_msg> {e} from portfolio.get_port_recent_ret_details')
+            df = pd.DataFrame()
+            df.index.name = 'datetime'
+            data = {
+                '组合收益':df,
+                '组合收益描述':[],
+                '资产价格比收益':df,
+                '资产价格比收益描述':[],
+                '个基近期收益':df,
+            }
+            return data
\ No newline at end of file
