@@ -218,4 +218,6 @@ class StyleAnalysisProcessor:
 
 if __name__ == "__main__":
     data_helper = DerivedDataHelper()
-    StyleAnalysisProcessor(data_helper, 'csi800').process('2005-01-01', '2014-12-31')
+    for universe in ('hs300', 'csi800', 'all'):
+        sap = StyleAnalysisProcessor(data_helper, universe)
+        sap.process('20201026', '20201026')
