@@ -1045,7 +1045,7 @@ class Calculator:
             }
             _dic = {
                 'pb_mrq':'PB',
-                'pe_ttm':'PE',
+                'pe_ttm_nn':'PE',
                 'roe':'ROE',
                 'ps_ttm':'PS',
                 'dy':'股息率',
@@ -1287,7 +1287,9 @@ class Calculator:
     def get_product_rolling_beta(fund_navs, windows=20, period='周度', index_id='hs300'):
         try:
             if fund_navs.empty:
-                return pandas.DataFrame()
+                df = pandas.DataFrame()
+                df.index.name = 'datetime'
+                return df
             if period == '日度':
                 rule = '1D'
                 if windows <= 2:
