@@ -366,7 +366,7 @@ class CalculatorBase:
                 period_1y = CalculatorBase.TRADING_DAYS_PER_YEAR
                 risk_free_per_period = risk_free_rate / CalculatorBase.TRADING_DAYS_PER_YEAR
                 dts = BasicDataApi().get_trading_day_list(start_date=sr.index[0], end_date=sr.index[-1]).datetime
-                sr_annual_vol = sr.reindex(dts).ffill()
+                sr_annual_vol = sr.reindex(sr.index.union(dts)).sort_index().ffill().reindex(dts)
                 sr_ret_annual_vol = sr_annual_vol.pct_change(1).iloc[1:]
             elif frequency == '1W':
                 period_1m = 4
@@ -400,7 +400,7 @@ class CalculatorBase:
             if frequency == '1D':
                 annual_vol = sr_ret_annual_vol.std(ddof=1) * np.sqrt(period_1y)
             else:
-                annual_vol = sr.std(ddof=1) * np.sqrt(period_1y)
+                annual_vol = sr.pct_change(1).iloc[1:].std(ddof=1) * np.sqrt(period_1y)
             _begin_date = sr.index[0]
             recent_ret_res = CalculatorBase.calc_recent_ret(dates=dates,values=values,frequency='1D')
             recent_year_ret = recent_ret_res['recent_natural_year_ret']
@@ -446,7 +446,7 @@ class CalculatorBase:
             ervar = (annual_ret - risk_free_rate) / var
             skew_value = skew(sr_ret)
             kurtosis_value = kurtosis(sr_ret)        
-            sr_ret_monthly = CalculatorBase.data_resample_monthly_ret(sr_ret).dropna()
+            sr_ret_monthly = CalculatorBase.data_resample_monthly_ret(df=sr_ret, min_count=1).dropna()
             raise_month_num = sr_ret_monthly[sr_ret_monthly>0].shape[0]
             drop_month_num = sr_ret_monthly[sr_ret_monthly<0].shape[0]
 
@@ -481,9 +481,11 @@ class CalculatorBase:
                 _index = sorted(sr_ret.index.intersection(sr_benchmark_ret.index))
                 sr_ret = sr_ret.reindex(_index)
                 sr_benchmark_ret = sr_benchmark_ret.reindex(_index)
-                sr_ret_weekly = CalculatorBase.data_resample_weekly_ret(sr_ret).dropna()
-                sr_benchmark_ret_weekly = CalculatorBase.data_resample_weekly_ret(sr_benchmark_ret).dropna()
-                sr_benchmark_ret_monthly = CalculatorBase.data_resample_monthly_ret(sr_benchmark_ret).dropna()
+                sr_ret_weekly = CalculatorBase.data_resample_weekly_ret(df=sr_ret,min_count=1).dropna()
+                sr_benchmark_ret_weekly = CalculatorBase.data_resample_weekly_ret(df=sr_benchmark_ret, min_count=1).dropna()
+                sr_benchmark_ret_monthly = CalculatorBase.data_resample_monthly_ret(df=sr_benchmark_ret, min_count=1).dropna()
+                
+                
                 win_rate = np.mean((sr_ret > sr_benchmark_ret) * 1)
                 win_rate_0 = np.mean((sr_ret > 0) * 1)
                 win_rate_weekly = np.mean((sr_ret_weekly > sr_benchmark_ret_weekly) * 1)
@@ -533,6 +535,8 @@ class CalculatorBase:
                 sr_excess_value = sr_excess_value.sort_index()
                 mdd_part =  sr_excess_value / sr_excess_value.rolling(window=sr_excess_value.shape[0], min_periods=1).max()
                 excess_mdd = 1 - mdd_part.min()
+                
+                
                 if np.isnan(excess_mdd):
                     excess_mdd_date2 = None
                     excess_mdd_date1 = None
